#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan  2 14:53:20 2023

@author: lpeurey

Manage  the importation of eaf 2022 annotation campaign
custom converter to import properly
"""
import glob
from pathlib import Path

import pandas as pd
import pympi
from collections import defaultdict
import os

from ChildProject.projects import ChildProject
from ChildProject.annotations import AnnotationManager

SPEAKER_ID_TO_TYPE = defaultdict(
        lambda: "NA",
        {
            "CHI": "CHI",
            "FEM": "FEM",
            "MAL": "MAL",
            "OCH": "OCH",
        },
    )

VCM_MAPPING = {
        'A':'N',
        'P':'N',
        'W':'C',
        'V':'C',
        'L':'L',
        'Y':'Y',
        'U':'U',
        }
XDS_MAPPING = {
        'T':'T',
        'C':'C',
        'B':'A,C',
        'A':'A',
        'P':'P',
        'O':'O',
        'U':'U',
        }

BP_EAFS = ['77033_5/77033_5.eaf']
BP_RECS = ['77033_5/77033_5.WAV']
def convert(filename: str, filter=None, **kwargs) -> pd.DataFrame:

    eaf = pympi.Elan.Eaf(filename)
    
    segments = {}
    for tier_name in eaf.tiers:
        annotations = eaf.tiers[tier_name][0]
    
        if (
            tier_name not in SPEAKER_ID_TO_TYPE
            and len(annotations) > 0
        ):
            print(
                "warning: unknown tier '{}' will be ignored in '{}'".format(
                    tier_name, filename
                )
            )
            continue

        for aid in annotations:
            (start_ts, end_ts, value, svg_ref) = annotations[aid]
            (start_t, end_t) = (eaf.timeslots[start_ts], eaf.timeslots[end_ts])
    
            segment = {
                "segment_onset": int(round(start_t)),
                "segment_offset": int(round(end_t)),
                "speaker_id": tier_name,
                "speaker_type": SPEAKER_ID_TO_TYPE[tier_name],
                "vcm_type": "NA",
                "vcm_type_precise": "NA",
                "msc_type": "NA",
                "xds_type": "NA",
                "gra_type": "NA",
                "addressee": "NA",
            }
    
            segments[aid] = segment
    
    for tier_name in eaf.tiers:
        if "@" in tier_name:
            label, ref = tier_name.split("@")
        else:
            label, ref = tier_name, None
    
        reference_annotations = eaf.tiers[tier_name][1]
    
        if ref not in SPEAKER_ID_TO_TYPE:
            continue
    
        for aid in reference_annotations:
            (ann, value, prev, svg) = reference_annotations[aid]
    
            ann = aid
            parentTier = eaf.tiers[eaf.annotations[ann]]
            while (
                "PARENT_REF" in parentTier[2]
                and parentTier[2]["PARENT_REF"]
                and len(parentTier[2]) > 0
            ):
                ann = parentTier[1][ann][0]
                parentTier = eaf.tiers[eaf.annotations[ann]]
    
            if ann not in segments:
                print(
                    "warning: annotation '{}' not found in segments for '{}'".format(
                        ann, filename
                    )
                )
                continue
    
            segment = segments[ann]
    
            if value: #discard segments that have no label (kept NA)
                if label == "vcm":
                    segment["vcm_type"] = VCM_MAPPING[value]
                    segment["vcm_type_precise"] = value
                elif label == "msc":
                    segment["msc_type"] = value
                elif label == "gra":
                    segment["gra_type"] = value
                elif label == "xds":
                    segment["addressee"] = XDS_MAPPING[value]
                
    
    return pd.DataFrame(segments.values())

if __name__ == '__main__' :
    
    project = ChildProject('.')
    am = AnnotationManager(project)
    

        #tier_names = [x for x in eaf.tiers.keys() if x in SPEAKER_ID_TO_TYPE and len(eaf.tiers[x][0]) > 0]

    files = pd.DataFrame([
        {'raw_filename': f}
        for f in glob.glob('annotations/eaf_2023/*/raw/*/*.eaf') if f.split('raw/')[-1] in BP_EAFS
    ])

    files['time_seek'] = 0
    files['recording_filename'] = files['raw_filename'].apply(lambda x: x.split('raw/')[-1].split('.')[0]) + '.WAV'
    files['set'] = files['raw_filename'].str.extract(r"^annotations/(eaf_2023/[A-Za-z]{2})")
    files['raw_filename'] = files['raw_filename'].apply(os.path.relpath, start='./annotations/eaf_2023/ak/raw/')
    files['format'] = 'eaf'

    _files = []

    for f in files.to_dict(orient='records'):
        eaf = pympi.Elan.Eaf(Path('annotations')/ f['set'] / 'raw' / f['raw_filename'])
        portions = eaf.tiers['chunks'][0]

        for pid in portions:
            (start_ts, end_ts, value, svg_ref) = portions[pid]
            (start_t, end_t) = (eaf.timeslots[start_ts], eaf.timeslots[end_ts])

            # if value.upper() != 'Y':
            #    continue

            f['range_onset'] = start_t
            f['range_offset'] = end_t

            _files.append(pd.DataFrame([f]))

    files = pd.concat(_files)
    am.remove_set('eaf_2023/ak')
    am.import_annotations(files, threads=1, import_function = convert)
