import os
import datetime
import sys
import ChildProject
import numpy as np
import pandas as pd


def find_recording(row, df, cumulative_time):
    # select index of a range from cumulative_time which fully contains the segment onset and offset
    cumulative_time = np.insert(cumulative_time,0, [0])
    cumulative_time = [0] + cumulative_time

    for i in range(1, len(cumulative_time)):
        rng = range(cumulative_time[i-1], cumulative_time[i])
        if (row['segment_onset'] in rng) & \
                (row['segment_offset'] in rng):
            return df.reset_index(drop=True).iloc[i-1, :]['recording_filename']
    return np.nan

def sample(
        destination: str = '../sampler',
        length: int = 60000, # 1 minute
        period: int = 600000, # 10 minutes
        offset: int = 300000, # 5 minutes
        recordings: str = './metadata/recordings.csv',
    ):
    """Periodic sampling of a recording if there are multiple recordings per session.
    :param destination: segments destination, defaults to ./sampler
    :type destination: str
    :param length: length of each segment, in milliseconds, defaults 60000
    :type length: int
    :param period: spacing between two consecutive segments, in milliseconds, defaults to 600000
    :type period: int
    :param offset: offset of the first segment, in milliseconds, defaults to 300000
    :type offset: int
    :param recordings: path to recordings to sample from; if None, all recordings will be sampled, defaults to './metadata/recordings.csv'
    :type recordings: str, optional

    If sampled 30 seconds span over more than one recording, 5 minutes is subtracted from the segment_onset and offset until whole sampled period
    is within one recording. If 5-minute subtraction continues until there is less than 5 minutes of the session left, this sampling is dropped
    """

    recordings = pd.read_csv(recordings)

    assert "duration" in recordings.columns, """recordings duration was not found in the metadata"""

    try:
        recordings["duration"].fillna(0, inplace=True)
    except KeyError:
        raise KeyError("""recordings duration was not found in the metadata""")

    segments = pd.DataFrame(columns=['recording_filename', 'segment_onset', 'segment_offset'], dtype=object)

    for name, group in recordings.groupby(['child_id', 'date_iso']):
        group = group.reset_index(drop=True)
        cumulative_time = group['duration'].cumsum().to_numpy()

        onsets = np.arange(
                offset,
                cumulative_time[-1] - length,
                period + length,
        )

        try:
            assert onsets.size != 0
        except AssertionError:
            print("Recordings of participant {0}, "
                  "session {1} not sampled; "
                  "total duration is {2:.2f} minutes".format(group.iloc[-1]['child_id'],
                                                             group.iloc[-1]['date_iso'],
                                                             cumulative_time[-1]/60000))
            continue

        segment = pd.DataFrame(data=onsets, columns=['segment_onset'])
        segment["segment_onset"] = segment["segment_onset"].astype(int)
        segment["segment_offset"] = segment["segment_onset"] + length

        segment['recording_filename'] = segment.apply(lambda row: find_recording(row, group, cumulative_time), axis=1)

        while segment.isna().any(axis=1).sum() > 0:
            index = segment.isna().any(axis=1)
            segment[index] = segment[index].apply(lambda x: x-300000)

            try:
                assert (segment.loc[index, 'segment_onset'] <= 300000).sum() == 0

            except AssertionError:
                segment = segment.drop((segment.loc[index, 'segment_onset'] <= 300000).index, axis='index')
                segment = segment.reset_index(drop=True)
                print("Recordings of participant {0}, "
                      "session {1} not sampled; "
                      "5-minute shift failed".format(group.iloc[-1]['child_id'],
                                                     group.iloc[-1]['date_iso']))
                continue

            segment.loc[index, 'recording_filename'] = segment[index].apply(lambda row: find_recording(row, group, cumulative_time), axis=1)

        segments = pd.concat([segments, segment], ignore_index=True)

    segments.rename(
        columns={"recording_filename": "recording_filename",
                 "segment_onset": "segment_onset",
                 "segment_offset": "segment_offset"}, inplace=True
    )

    date = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")

    os.makedirs(destination, exist_ok=True)
    segments_path = os.path.join(destination, "segments_{}.csv".format(date))
    #segments = segments.rename(columns={'recording_filename': 'dataset_recording_filename'})
    segments['mp3_recording_filename'] = segments['recording_filename'].split(".")[0] + ".mp3"

    segments[
        set(segments.columns)
        & {"recording_filename", "segment_onset", "segment_offset"}
        ].to_csv(segments_path, index=False)
    print("exported sampled segments to {}".format(segments_path))
#sample(recordings='../metadata/recordings.csv')

# segments = pd.read_csv('../sampler/segments_20240319_155608.csv')
# print(segments)
# segments = segments.rename(columns={'recording_filename': 'dataset_recording_filename'})
# segments['recording_filename'] = segments['dataset_recording_filename'].apply(lambda x: x.split(".")[0] + ".mp3")
# print(segments)
# segments.to_csv('../sampler/segments_20240319_155608.csv')
if __name__ == "__main__":
    args = sys.argv
    globals()[args[1]](*args[2:])
